package it.cavallium;

import java.util.Objects;
import java.util.StringJoiner;

public class ItemUpdate<T> {
	private final boolean removed;
	private final T item;

	public ItemUpdate(boolean removed, T item) {
		this.removed = removed;
		this.item = item;
	}

	public boolean isRemoved() {
		return removed;
	}

	public T getItem() {
		return item;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		ItemUpdate<?> that = (ItemUpdate<?>) o;

		if (removed != that.removed) {
			return false;
		}
		return Objects.equals(item, that.item);
	}

	@Override
	public int hashCode() {
		int result = (removed ? 1 : 0);
		result = 31 * result + (item != null ? item.hashCode() : 0);
		return result;
	}

	@Override
	public String toString() {
		return new StringJoiner(", ", ItemUpdate.class.getSimpleName() + "[", "]")
				.add("removed=" + removed)
				.add("item=" + item)
				.toString();
	}
}
