package it.cavallium;

import java.util.Objects;
import java.util.StringJoiner;

public class SupergroupInfo {

	private final BaseChatInfo baseChatInfo;
	private final boolean canRestrictMembers;
	private final boolean canInviteUsers;

	public SupergroupInfo(BaseChatInfo baseChatInfo, boolean canRestrictMembers, boolean canInviteUsers) {
		this.baseChatInfo = baseChatInfo;
		this.canRestrictMembers = canRestrictMembers;
		this.canInviteUsers = canInviteUsers;
	}

	public BaseChatInfo getBaseChatInfo() {
		return baseChatInfo;
	}

	public boolean canInviteUsers() {
		return canInviteUsers;
	}

	public boolean canRestrictMembers() {
		return canRestrictMembers;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		SupergroupInfo that = (SupergroupInfo) o;

		if (canRestrictMembers != that.canRestrictMembers) {
			return false;
		}
		if (canInviteUsers != that.canInviteUsers) {
			return false;
		}
		return Objects.equals(baseChatInfo, that.baseChatInfo);
	}

	@Override
	public int hashCode() {
		int result = baseChatInfo != null ? baseChatInfo.hashCode() : 0;
		result = 31 * result + (canRestrictMembers ? 1 : 0);
		result = 31 * result + (canInviteUsers ? 1 : 0);
		return result;
	}

	@Override
	public String toString() {
		return new StringJoiner(", ", SupergroupInfo.class.getSimpleName() + "[", "]")
				.add("baseChatInfo=" + baseChatInfo)
				.add("canRestrictMembers=" + canRestrictMembers)
				.add("canInviteUsers=" + canInviteUsers)
				.toString();
	}
}
